package org.eclipse.equinox.launcher;

import com.jiuqi.dna.launcher.Constants;
import com.jiuqi.dna.launcher.ServerDaemon;
import com.jiuqi.util.ArrayUtils;
import com.jiuqi.util.BundleUtils;
import com.jiuqi.util.StringUtil;
import java.io.BufferedOutputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLDecoder;
import java.net.URLStreamHandlerFactory;
import java.security.AllPermission;
import java.security.CodeSource;
import java.security.Permission;
import java.security.PermissionCollection;
import java.security.Policy;
import java.security.ProtectionDomain;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.Locale;
import java.util.NoSuchElementException;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

/* loaded from: org_eclipse_equinox_launcher.jar:org/eclipse/equinox/launcher/Main.class */
public class Main implements Constants {
    public static final String SPLASH_HANDLE = "org.eclipse.equinox.launcher.splash.handle";
    public static final String SPLASH_LOCATION = "org.eclipse.equinox.launcher.splash.location";
    private static final String FRAMEWORK = "-framework";
    private static final String INSTALL = "-install";
    private static final String INITIALIZE = "-initialize";
    private static final String VM = "-vm";
    private static final String VMARGS = "-vmargs";
    private static final String DEBUG = "-debug";
    private static final String DEV = "-dev";
    private static final String CONFIGURATION = "-configuration";
    private static final String NOSPLASH = "-nosplash";
    private static final String SHOWSPLASH = "-showsplash";
    private static final String EXITDATA = "-exitdata";
    private static final String NAME = "-name";
    private static final String LAUNCHER = "-launcher";
    private static final String LIBRARY = "--launcher.library";
    private static final String NL = "-nl";
    private static final String ENDSPLASH = "-endsplash";
    private static final String SPLASH_IMAGE = "splash.bmp";
    private static final String CLEAN = "-clean";
    private static final String NOEXIT = "-noExit";
    private static final String OS = "-os";
    private static final String WS = "-ws";
    private static final String REDIRECT = "-redirect";
    private static final String ARCH = "-arch";
    private static final String STARTUP = "-startup";
    private static final String STARTER = "org.eclipse.core.runtime.adaptor.EclipseStarter";
    private static final String PLATFORM_URL = "platform:/base/";
    private static final String ECLIPSE_PROPERTIES = "eclipse.properties";
    private static final String FILE_SCHEME = "file:";
    protected static final String REFERENCE_SCHEME = "reference:";
    protected static final String JAR_SCHEME = "jar:";
    private static final String CONFIG_DIR = "configuration/";
    private static final String CONFIG_FILE = "config.ini";
    private static final String CONFIG_FILE_TEMP_SUFFIX = ".tmp";
    private static final String CONFIG_FILE_BAK_SUFFIX = ".bak";
    private static final String ECLIPSE = "eclipse";
    private static final String PRODUCT_SITE_MARKER = ".eclipseproduct";
    private static final String PRODUCT_SITE_ID = "id";
    private static final String PRODUCT_SITE_VERSION = "version";
    private static final String PROP_USER_HOME = "user.home";
    private static final String PROP_USER_DIR = "user.dir";
    private static final String PROP_INSTALL_AREA = "osgi.install.area";
    private static final String PROP_OS_TYPE = "os.type";
    private static final String PROP_REDIRECT_OUT = "out.redirect";
    private static final String PROP_CONSOLE_BLOCK = "osgi.console.blockOnReady";
    private static final String PROP_CONFIG_AREA = "osgi.configuration.area";
    private static final String PROP_CONFIG_AREA_DEFAULT = "osgi.configuration.area.default";
    private static final String PROP_BASE_CONFIG_AREA = "osgi.baseConfiguration.area";
    private static final String PROP_SHARED_CONFIG_AREA = "osgi.sharedConfiguration.area";
    private static final String PROP_CONFIG_CASCADED = "osgi.configuration.cascaded";
    protected static final String PROP_FRAMEWORK = "osgi.framework";
    private static final String PROP_SPLASHPATH = "osgi.splashPath";
    private static final String PROP_SPLASHLOCATION = "osgi.splashLocation";
    private static final String PROP_CLASSPATH = "osgi.frameworkClassPath";
    private static final String PROP_EXTENSIONS = "osgi.framework.extensions";
    private static final String PROP_FRAMEWORK_SYSPATH = "osgi.syspath";
    private static final String PROP_FRAMEWORK_SHAPE = "osgi.framework.shape";
    private static final String PROP_LOGFILE = "osgi.logfile";
    private static final String PROP_REQUIRED_JAVA_VERSION = "osgi.requiredJavaVersion";
    private static final String PROP_PARENT_CLASSLOADER = "osgi.parentClassloader";
    private static final String PROP_FRAMEWORK_PARENT_CLASSLOADER = "osgi.frameworkParentClassloader";
    private static final String PROP_NL = "osgi.nl";
    static final String PROP_NOSHUTDOWN = "osgi.noShutdown";
    private static final String PROP_DEBUG = "osgi.debug";
    private static final String PROP_EXITCODE = "eclipse.exitcode";
    private static final String PROP_EXITDATA = "eclipse.exitdata";
    private static final String PROP_VM = "eclipse.vm";
    private static final String PROP_VMARGS = "eclipse.vmargs";
    private static final String PROP_COMMANDS = "eclipse.commands";
    private static final String PROP_ECLIPSESECURITY = "eclipse.security";
    private static final String NONE = "@none";
    private static final String NO_DEFAULT = "@noDefault";
    private static final String USER_HOME = "@user.home";
    private static final String USER_DIR = "@user.dir";
    private static final String PARENT_CLASSLOADER_EXT = "ext";
    private static final String PARENT_CLASSLOADER_BOOT = "boot";
    private static final String PARENT_CLASSLOADER_CURRENT = "current";
    protected static final String SESSION = "!SESSION";
    protected static final String ENTRY = "!ENTRY";
    protected static final String MESSAGE = "!MESSAGE";
    protected static final String STACK = "!STACK";
    protected static final int ERROR = 4;
    private static final String prefix2 = "reference:file:..";
    private static final String SUBFIX_START = "@start";
    private static final String PARENT_CLASSLOADER_APP = "app";
    private static final String[] bundleDirs = {PARENT_CLASSLOADER_APP, "dna", "thr", "lib", "bundles"};
    private static final String OSGI = "org.eclipse.osgi";
    protected static final String PLUGIN_ID = "com.jiuqi.dna.launcher";
    private static final String[] notLoadBundles = {OSGI, PLUGIN_ID};
    protected boolean debug = false;
    protected String bootLocation = null;
    protected URL installLocation = null;
    protected URL configurationLocation = null;
    protected String parentConfigurationLocation = null;
    protected String framework = OSGI;
    protected String devClassPath = null;
    private Properties devClassPathProps = null;
    protected boolean inDevelopmentMode = false;
    protected String os = null;
    protected String ws = null;
    protected String arch = null;
    private String library = null;
    private String exitData = null;
    private String vm = null;
    private String[] vmargs = null;
    private String[] commands = null;
    String[] extensionPaths = null;
    JNIBridge bridge = null;
    private boolean showSplash = false;
    private String splashLocation = null;
    private String endSplash = null;
    private boolean initialize = false;
    private boolean splashDown = false;
    private final Thread splashHandler = new SplashHandler();
    protected File logFile = null;
    protected BufferedWriter log = null;
    protected boolean newSession = true;

    /* loaded from: org_eclipse_equinox_launcher.jar:org/eclipse/equinox/launcher/Main$SplashHandler.class */
    public final class SplashHandler extends Thread {
        public SplashHandler() {
        }

        @Override // java.lang.Thread, java.lang.Runnable
        public void run() {
            Main.this.takeDownSplash();
        }

        public void updateSplash() {
            if (Main.this.bridge != null) {
                Main.this.bridge.updateSplash();
            }
        }
    }

    /* JADX INFO: Access modifiers changed from: package-private */
    /* loaded from: org_eclipse_equinox_launcher.jar:org/eclipse/equinox/launcher/Main$Identifier.class */
    public static class Identifier {
        private static final String DELIM = ". _-";
        private int major;
        private int minor;
        private int service;

        Identifier(int major, int minor, int service) {
            this.major = major;
            this.minor = minor;
            this.service = service;
        }

        Identifier(String versionString) {
            StringTokenizer tokenizer = new StringTokenizer(versionString, DELIM);
            if (tokenizer.hasMoreTokens()) {
                this.major = Integer.parseInt(tokenizer.nextToken());
            }
            if (tokenizer.hasMoreTokens()) {
                this.minor = Integer.parseInt(tokenizer.nextToken());
            }
            try {
                if (tokenizer.hasMoreTokens()) {
                    this.service = Integer.parseInt(tokenizer.nextToken());
                }
            } catch (NumberFormatException unused) {
            }
        }

        boolean isGreaterEqualTo(Identifier minimum) {
            if (this.major < minimum.major) {
                return false;
            }
            if (this.major > minimum.major) {
                return true;
            }
            if (this.minor < minimum.minor) {
                return false;
            }
            return this.minor > minimum.minor || this.service >= minimum.service;
        }
    }

    private String getWS() {
        if (this.ws != null) {
            return this.ws;
        }
        String os = getOS();
        if (os.equals("win32")) {
            return "win32";
        }
        if (os.equals("linux")) {
            return "gtk";
        }
        if (os.equals("macosx")) {
            return "carbon";
        }
        if (os.equals("hpux") || os.equals("aix") || os.equals("solaris")) {
            return "motif";
        }
        if (os.equals("qnx")) {
            return "photon";
        }
        return "unknown";
    }

    private String getOS() {
        if (this.os != null) {
            return this.os;
        }
        String osName = System.getProperties().getProperty("os.name");
        if (osName.regionMatches(true, 0, "win32", 0, 3)) {
            return "win32";
        }
        if (osName.equalsIgnoreCase("SunOS")) {
            return "solaris";
        }
        if (osName.equalsIgnoreCase("Linux")) {
            return "linux";
        }
        if (osName.equalsIgnoreCase("QNX")) {
            return "qnx";
        }
        if (osName.equalsIgnoreCase("AIX")) {
            return "aix";
        }
        if (osName.equalsIgnoreCase("HP-UX")) {
            return "hpux";
        }
        if (osName.regionMatches(true, 0, "Mac OS", 0, "Mac OS".length())) {
            return "macosx";
        }
        return "unknown";
    }

    private String getArch() {
        if (this.arch != null) {
            return this.arch;
        }
        String name = System.getProperties().getProperty("os.arch");
        if (name.equalsIgnoreCase("i386")) {
            return "x86";
        }
        if (name.equalsIgnoreCase("amd64")) {
            return "x86_64";
        }
        return name;
    }

    private void setupJNI(URL[] defaultPath) {
        String fragment;
        String libPath = null;
        if (this.library != null) {
            File lib = new File(this.library);
            if (lib.isDirectory()) {
                libPath = searchFor(ECLIPSE, lib.getAbsolutePath());
            } else if (lib.exists()) {
                libPath = lib.getAbsolutePath();
            }
        }
        if (libPath == null) {
            String fragmentOS = getOS();
            StringBuffer buffer = new StringBuffer(PLUGIN_ID);
            buffer.append('.');
            buffer.append(getWS());
            buffer.append('.');
            buffer.append(fragmentOS);
            if (!fragmentOS.equals("macosx")) {
                buffer.append('.');
                buffer.append(getArch());
            }
            String fragmentName = buffer.toString();
            String fragment2 = null;
            if (this.inDevelopmentMode) {
                String devPathList = this.devClassPathProps.getProperty(PLUGIN_ID);
                String[] locations = getArrayFromList(devPathList);
                if (locations.length > 0) {
                    File location = new File(locations[0]);
                    if (location.isAbsolute()) {
                        fragment2 = searchFor(fragmentName, location.getParent());
                        if (fragment2 != null) {
                            libPath = searchFor(ECLIPSE, fragment2);
                        }
                    }
                }
            }
            if (libPath == null && this.bootLocation != null && defaultPath != null && defaultPath.length > 0) {
                for (int i = defaultPath.length - 1; i >= 0 && libPath == null; i--) {
                    File entryFile = new File(defaultPath[i].getFile());
                    String dir = entryFile.getParent();
                    if (this.inDevelopmentMode) {
                        String devDir = String.valueOf(dir) + "/" + PLUGIN_ID + "/fragments";
                        fragment2 = searchFor(fragmentName, devDir);
                    }
                    if (fragment2 == null) {
                        fragment2 = searchFor(fragmentName, dir);
                    }
                    if (fragment2 != null) {
                        libPath = searchFor(ECLIPSE, fragment2);
                    }
                }
            }
            if (libPath == null) {
                URL install = getInstallLocation();
                String location2 = String.valueOf(install.getFile()) + "/bundles/";
                if (new File(location2).exists()) {
                    fragment = searchFor(fragmentName, location2);
                } else {
                    fragment = searchFor(fragmentName, String.valueOf(location2) + "/thr/");
                }
                if (fragment != null) {
                    libPath = searchFor(ECLIPSE, fragment);
                }
            }
        }
        this.library = libPath;
        if (this.library != null) {
            this.bridge = new JNIBridge(this.library);
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public void basicRun(String[] args) throws Exception {
        System.getProperties().put("eclipse.startTime", Long.toString(System.currentTimeMillis()));
        String[] args2 = {OS, System.getProperty("os.name"), "-console"};
        this.commands = args2;
        String[] passThruArgs = processCommandLine(args2);
        if (!this.debug) {
            this.debug = System.getProperty(PROP_DEBUG) != null;
        }
        setupVMProperties();
        getInstallLocation();
        String installPath = System.getProperty(PROP_INSTALL_AREA);
        if (installPath != null) {
            URL url = new URL(installPath);
            File f = new File(url.getFile());
            installPath = f.getAbsolutePath();
            System.setProperty("com.jiuqi.dna.rootpath", installPath);
            System.getProperties().put(PROP_CONFIG_AREA, String.valueOf(installPath) + "/work");
        }
        processConfiguration2();
        if (args.length > 0 && "stop".equals(args[0])) {
            ServerDaemon.stopServer();
        } else if (args.length > 0 && "restart".equals(args[0])) {
            ServerDaemon.restartServer();
        } else {
            ServerDaemon.startServer();
        }
        if (Boolean.getBoolean(PROP_REDIRECT_OUT)) {
            IORedirecter.redirectIO(installPath);
        }
        if (Boolean.getBoolean(PROP_CONSOLE_BLOCK)) {
            System.setProperty(PROP_CONSOLE_BLOCK, "");
        }
        URL[] bootPath = getBootPath(this.bootLocation);
        setupJNI(bootPath);
        if (!checkVersion(System.getProperty("java.version"), System.getProperty(PROP_REQUIRED_JAVA_VERSION))) {
            return;
        }
        setSecurityPolicy(bootPath);
        handleSplash(bootPath);
        beforeFwkInvocation();
        invokeFramework(passThruArgs, bootPath);
    }

    protected void beforeFwkInvocation() {
    }

    private final void processConfiguration2() {
        System.setProperty("osgi.clean", "true");
        System.setProperty("org.eclipse.equinox.http.jetty.autostart", "false");
        System.setProperty("POOL_MAX", "100000");
        System.setProperty(PROP_NOSHUTDOWN, "true");
        StringBuffer initBundles = new StringBuffer();
        String serverRoot = System.getProperty("com.jiuqi.dna.rootpath");
        File f = new File(serverRoot);
        String serverRootPath = StringUtil.formatPath(f.getPath(), false);
        if (serverRootPath.endsWith("/")) {
            serverRootPath = serverRootPath.substring(0, serverRootPath.length() - 1);
        }
        for (String bundleDir : bundleDirs) {
            File fBundleDir = new File(serverRoot, bundleDir);
            doStruct(fBundleDir, initBundles, serverRootPath);
        }
        if (Boolean.getBoolean("launcher.debug")) {
            System.out.println(initBundles.toString());
        }
        System.setProperty("osgi.bundles", initBundles.toString());
    }

    private final void doStruct(File fBundleDir, StringBuffer configBuffer, String serverRootPath) {
        String filePath;
        if (BundleUtils.isBundle(fBundleDir)) {
            String bundleId = BundleUtils.getBundleId(fBundleDir);
            if (isNotLoadBundles(bundleId)) {
                return;
            }
            String filePath2 = StringUtil.formatPath(fBundleDir.getPath(), false).substring(serverRootPath.length());
            if (canStartBundle(fBundleDir)) {
                filePath = prefix2 + filePath2 + SUBFIX_START;
            } else {
                filePath = prefix2 + filePath2;
            }
            configBuffer.append(filePath);
            configBuffer.append(",");
            return;
        }
        if (fBundleDir.isDirectory()) {
            for (File f : fBundleDir.listFiles()) {
                doStruct(f, configBuffer, serverRootPath);
            }
        }
    }

    private final boolean isNotLoadBundles(String bundleId) {
        return ArrayUtils.contains(notLoadBundles, bundleId);
    }

    private boolean canStartBundle(File bundleFile) {
        return !BundleUtils.isFragmentBundle(bundleFile);
    }

    protected void setSecurityPolicy(URL[] bootPath) {
        String eclipseSecurity = System.getProperty(PROP_ECLIPSESECURITY);
        if (eclipseSecurity != null) {
            SecurityManager sm = System.getSecurityManager();
            boolean setSM = false;
            if (sm == null) {
                if (eclipseSecurity.length() < 1) {
                    eclipseSecurity = "java.lang.SecurityManager";
                }
                try {
                    Class clazz = Class.forName(eclipseSecurity);
                    sm = (SecurityManager) clazz.newInstance();
                    setSM = true;
                } catch (Throwable unused) {
                    System.getProperties().put("java.security.manager", eclipseSecurity);
                }
            }
            ProtectionDomain domain = Main.class.getProtectionDomain();
            CodeSource source = null;
            if (domain != null) {
                source = Main.class.getProtectionDomain().getCodeSource();
            }
            if (domain == null || source == null) {
                log("Can not automatically set the security manager. Please use a policy file.");
                return;
            }
            URL[] rootURLs = new URL[bootPath.length + 1];
            rootURLs[0] = source.getLocation();
            System.arraycopy(bootPath, 0, rootURLs, 1, bootPath.length);
            Policy eclipsePolicy = new EclipsePolicy(Policy.getPolicy(), rootURLs);
            Policy.setPolicy(eclipsePolicy);
            if (setSM) {
                System.setSecurityManager(sm);
            }
        }
    }

    private void invokeFramework(String[] passThruArgs, URL[] bootPath) throws ClassNotFoundException, NoSuchMethodException, IllegalAccessException, Error, Exception, InvocationTargetException {
        String type = System.getProperty(PROP_FRAMEWORK_PARENT_CLASSLOADER, System.getProperty(PROP_PARENT_CLASSLOADER, PARENT_CLASSLOADER_BOOT));
        ClassLoader parent = null;
        if (PARENT_CLASSLOADER_APP.equalsIgnoreCase(type)) {
            parent = ClassLoader.getSystemClassLoader();
        } else if (PARENT_CLASSLOADER_EXT.equalsIgnoreCase(type)) {
            ClassLoader appCL = ClassLoader.getSystemClassLoader();
            if (appCL != null) {
                parent = appCL.getParent();
            }
        } else if (PARENT_CLASSLOADER_CURRENT.equalsIgnoreCase(type)) {
            parent = getClass().getClassLoader();
        }
        URLClassLoader loader = new StartupClassLoader(bootPath, parent);
        Class clazz = loader.loadClass(STARTER);
        Method method = clazz.getDeclaredMethod("run", String[].class, Runnable.class);
        try {
            method.invoke(clazz, passThruArgs, this.splashHandler);
        } catch (InvocationTargetException e) {
            if (e.getTargetException() instanceof Error) {
                throw ((Error) e.getTargetException());
            }
            if (e.getTargetException() instanceof Exception) {
                throw ((Exception) e.getTargetException());
            }
            throw e;
        }
    }

    private boolean checkVersion(String availableVersion, String requiredVersion) {
        if (requiredVersion == null || availableVersion == null) {
            return true;
        }
        try {
            Identifier required = new Identifier(requiredVersion);
            Identifier available = new Identifier(availableVersion);
            boolean compatible = available.isGreaterEqualTo(required);
            if (!compatible) {
                System.getProperties().put(PROP_EXITCODE, "14");
                System.getProperties().put(PROP_EXITDATA, "<title>Incompatible JVM</title>Version " + availableVersion + " of the JVM is not suitable for this product. Version: " + requiredVersion + " or greater is required.");
            }
            return compatible;
        } catch (NumberFormatException unused) {
            return true;
        } catch (SecurityException unused2) {
            return true;
        }
    }

    protected String decode(String urlString) {
        try {
            Method method = URLDecoder.class.getDeclaredMethod("decode", String.class, String.class);
            if (urlString.indexOf(43) >= 0) {
                int len = urlString.length();
                StringBuffer buf = new StringBuffer(len);
                for (int i = 0; i < len; i++) {
                    char c = urlString.charAt(i);
                    if (c == '+') {
                        buf.append("%2B");
                    } else {
                        buf.append(c);
                    }
                }
                urlString = buf.toString();
            }
            Object result = method.invoke(null, urlString, "UTF-8");
            if (result != null) {
                return (String) result;
            }
        } catch (Exception unused) {
        }
        boolean replaced = false;
        byte[] encodedBytes = urlString.getBytes();
        int encodedLength = encodedBytes.length;
        byte[] decodedBytes = new byte[encodedLength];
        int decodedLength = 0;
        int i2 = 0;
        while (i2 < encodedLength) {
            byte b = encodedBytes[i2];
            if (b == 37) {
                if (i2 + 2 >= encodedLength) {
                    throw new IllegalArgumentException("Malformed URL (\"" + urlString + "\"): % must be followed by 2 digits.");
                }
                int i3 = i2 + 1;
                byte enc1 = encodedBytes[i3];
                i2 = i3 + 1;
                byte enc2 = encodedBytes[i2];
                b = (byte) ((hexToByte(enc1) << ERROR) + hexToByte(enc2));
                replaced = true;
            }
            int i4 = decodedLength;
            decodedLength++;
            decodedBytes[i4] = b;
            i2++;
        }
        if (!replaced) {
            return urlString;
        }
        try {
            return new String(decodedBytes, 0, decodedLength, "UTF-8");
        } catch (UnsupportedEncodingException unused2) {
            return new String(decodedBytes, 0, decodedLength);
        }
    }

    protected String[] getArrayFromList(String prop) {
        if (prop == null || prop.trim().equals("")) {
            return new String[0];
        }
        Vector list = new Vector();
        StringTokenizer tokens = new StringTokenizer(prop, ",");
        while (tokens.hasMoreTokens()) {
            String token = tokens.nextToken().trim();
            if (!token.equals("")) {
                list.addElement(token);
            }
        }
        return list.isEmpty() ? new String[0] : (String[]) list.toArray(new String[list.size()]);
    }

    private URL[] getDevPath(URL base) throws IOException {
        ArrayList result = new ArrayList(5);
        if (this.inDevelopmentMode) {
            addDevEntries(base, result, OSGI);
        }
        addBaseJars(base, result);
        return (URL[]) result.toArray(new URL[result.size()]);
    }

    URL constructURL(URL url, String name) {
        String externalForm = url.toExternalForm();
        if (externalForm.endsWith(".jar")) {
            try {
                return new URL(JAR_SCHEME + url + "!/" + name);
            } catch (MalformedURLException unused) {
            }
        }
        try {
            return new URL(url, name);
        } catch (MalformedURLException unused2) {
            return null;
        }
    }

    private void readFrameworkExtensions(URL base, ArrayList result) throws IOException {
        URL extensionURL;
        String qualifiedPath;
        String[] extensions = getArrayFromList(System.getProperties().getProperty(PROP_EXTENSIONS));
        String parent = new File(base.getFile()).getParent().toString();
        ArrayList extensionResults = new ArrayList(extensions.length);
        for (int i = 0; i < extensions.length; i++) {
            String path = searchFor(extensions[i], parent);
            if (path == null) {
                log("Could not find extension: " + extensions[i]);
            } else {
                if (this.debug) {
                    System.out.println("Loading extension: " + extensions[i]);
                }
                if (this.installLocation.getProtocol().equals("file")) {
                    extensionResults.add(path);
                    extensionURL = new File(path).toURL();
                } else {
                    extensionURL = new URL(this.installLocation.getProtocol(), this.installLocation.getHost(), this.installLocation.getPort(), path);
                }
                Properties extensionProperties = null;
                try {
                    extensionProperties = loadProperties(constructURL(extensionURL, ECLIPSE_PROPERTIES));
                } catch (IOException unused) {
                    if (this.debug) {
                        System.out.println("\teclipse.properties not found");
                    }
                }
                String extensionClassPath = null;
                if (extensionProperties != null) {
                    extensionClassPath = extensionProperties.getProperty(PROP_CLASSPATH);
                } else {
                    extensionProperties = new Properties();
                }
                String[] entries = (extensionClassPath == null || extensionClassPath.length() == 0) ? new String[]{""} : getArrayFromList(extensionClassPath);
                if (System.getProperty(PROP_CLASSPATH) == null) {
                    qualifiedPath = ".";
                } else {
                    qualifiedPath = "";
                }
                for (String str : entries) {
                    qualifiedPath = String.valueOf(qualifiedPath) + ", file:" + path + str;
                }
                extensionProperties.put(PROP_CLASSPATH, qualifiedPath);
                mergeProperties(System.getProperties(), extensionProperties);
                if (this.inDevelopmentMode) {
                    addDevEntries(extensionURL, result, extensions[i]);
                }
            }
        }
        this.extensionPaths = (String[]) extensionResults.toArray(new String[extensionResults.size()]);
    }

    private void addBaseJars(URL base, ArrayList result) throws IOException {
        URL url;
        String baseJarList = System.getProperty(PROP_CLASSPATH);
        if (baseJarList == null) {
            readFrameworkExtensions(base, result);
            baseJarList = System.getProperties().getProperty(PROP_CLASSPATH);
        }
        File fwkFile = new File(base.getFile());
        boolean fwkIsDirectory = fwkFile.isDirectory();
        if (fwkIsDirectory) {
            System.getProperties().put(PROP_FRAMEWORK_SHAPE, "folder");
        } else {
            System.getProperties().put(PROP_FRAMEWORK_SHAPE, "jar");
        }
        String fwkPath = new File(new File(base.getFile()).getParent()).getAbsolutePath();
        if (Character.isUpperCase(fwkPath.charAt(0))) {
            char[] chars = fwkPath.toCharArray();
            chars[0] = Character.toLowerCase(chars[0]);
            fwkPath = new String(chars);
        }
        System.getProperties().put(PROP_FRAMEWORK_SYSPATH, fwkPath);
        String[] baseJars = getArrayFromList(baseJarList);
        if (baseJars.length == 0) {
            if (!this.inDevelopmentMode && new File(base.getFile()).isDirectory()) {
                throw new IOException("Unable to initialize osgi.frameworkClassPath");
            }
            addEntry(base, result);
            return;
        }
        for (String string : baseJars) {
            try {
                if (string.equals(".")) {
                    addEntry(base, result);
                }
                if (string.startsWith(FILE_SCHEME)) {
                    url = new File(string.substring(5)).toURL();
                } else {
                    url = new URL(string);
                }
                addEntry(url, result);
            } catch (MalformedURLException unused) {
                addEntry(new URL(base, string), result);
            }
        }
    }

    protected void addEntry(URL url, List result) {
        if (new File(url.getFile()).exists()) {
            result.add(url);
        }
    }

    private void addDevEntries(URL base, List result, String symbolicName) throws MalformedURLException {
        URL url;
        if (this.devClassPathProps == null) {
            return;
        }
        String devPathList = this.devClassPathProps.getProperty(symbolicName);
        if (devPathList == null) {
            devPathList = this.devClassPathProps.getProperty("*");
        }
        String[] locations = getArrayFromList(devPathList);
        for (String location : locations) {
            File path = new File(location);
            if (path.isAbsolute()) {
                url = path.toURL();
            } else {
                char lastChar = location.charAt(location.length() - 1);
                if (location.endsWith(".jar") || lastChar == '/' || lastChar == '\\') {
                    url = new URL(base, location);
                } else {
                    url = new URL(base, String.valueOf(location) + "/");
                }
            }
            URL url2 = url;
            addEntry(url2, result);
        }
    }

    protected URL[] getBootPath(String base) throws IOException {
        URL url;
        if (base != null) {
            url = buildURL(base, true);
        } else {
            URL url2 = getInstallLocation();
            String path = null;
            File dir = new File(url2.getFile(), "bundles");
            if (dir.exists()) {
                path = dir.getAbsolutePath();
            } else {
                File dir2 = new File(url2.getFile(), "thr");
                if (dir2.exists()) {
                    path = dir2.getAbsolutePath();
                }
            }
            String path2 = searchFor(this.framework, path);
            if (path2 == null) {
                throw new RuntimeException("Could not find framework");
            }
            if (url2.getProtocol().equals("file")) {
                url = new File(path2).toURL();
            } else {
                url = new URL(url2.getProtocol(), url2.getHost(), url2.getPort(), path2);
            }
        }
        if (System.getProperty(PROP_FRAMEWORK) == null) {
            System.getProperties().put(PROP_FRAMEWORK, url.toExternalForm());
        }
        if (this.debug) {
            System.out.println("Framework located:\n    " + url.toExternalForm());
        }
        URL[] result = getDevPath(url);
        if (this.debug) {
            System.out.println("Framework classpath:");
            for (URL url3 : result) {
                System.out.println("    " + url3.toExternalForm());
            }
        }
        return result;
    }

    protected String searchFor(String target, String start) {
        return searchFor(target, null, start);
    }

    protected String searchFor(String target, String targetSuffix, String start) {
        String[] candidates = new File(start).list();
        if (candidates == null) {
            return null;
        }
        ArrayList matches = new ArrayList(2);
        for (int i = 0; i < candidates.length; i++) {
            if (candidates[i].equals(target) || candidates[i].startsWith(String.valueOf(target) + "_")) {
                matches.add(candidates[i]);
            }
        }
        String[] names = (String[]) matches.toArray(new String[matches.size()]);
        int result = findMax(names);
        if (result == -1) {
            return null;
        }
        File candidate = new File(start, names[result]);
        return String.valueOf(candidate.getAbsolutePath().replace(File.separatorChar, '/')) + (candidate.isDirectory() ? "/" : "");
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public int findMax(String[] candidates) {
        int result = -1;
        Object maxVersion = null;
        for (int i = 0; i < candidates.length; i++) {
            String name = candidates[i];
            String version = "";
            int index = name.indexOf(95);
            if (index != -1) {
                version = name.substring(index + 1);
            }
            Object currentVersion = getVersionElements(version);
            if (maxVersion == null) {
                result = i;
                maxVersion = currentVersion;
            } else if (compareVersion((Object[]) maxVersion, (Object[]) currentVersion) < 0) {
                result = i;
                maxVersion = currentVersion;
            }
        }
        return result;
    }

    private int compareVersion(Object[] left, Object[] right) {
        int result = ((Integer) left[0]).compareTo((Integer) right[0]);
        if (result != 0) {
            return result;
        }
        int result2 = ((Integer) left[1]).compareTo((Integer) right[1]);
        if (result2 != 0) {
            return result2;
        }
        int result3 = ((Integer) left[2]).compareTo((Integer) right[2]);
        if (result3 != 0) {
            return result3;
        }
        return ((String) left[3]).compareTo((String) right[3]);
    }

    private Object[] getVersionElements(String version) {
        if (version.endsWith(".jar")) {
            version = version.substring(0, version.length() - ERROR);
        }
        Object[] result = new Object[ERROR];
        result[0] = new Integer(0);
        result[1] = new Integer(0);
        result[2] = new Integer(0);
        result[3] = "";
        StringTokenizer t = new StringTokenizer(version, ".");
        int i = 0;
        while (t.hasMoreTokens() && i < ERROR) {
            String token = t.nextToken();
            if (i < 3) {
                try {
                    int i2 = i;
                    i++;
                    result[i2] = new Integer(token);
                } catch (Exception unused) {
                }
            } else {
                int i3 = i;
                i++;
                result[i3] = token;
            }
        }
        return result;
    }

    private static URL buildURL(String spec, boolean trailingSlash) {
        if (spec == null) {
            return null;
        }
        boolean isFile = spec.startsWith(FILE_SCHEME);
        try {
            if (isFile) {
                File toAdjust = new File(spec.substring(5));
                if (toAdjust.isDirectory()) {
                    return adjustTrailingSlash(toAdjust.toURL(), trailingSlash);
                }
                return toAdjust.toURL();
            }
            return new URL(spec);
        } catch (MalformedURLException unused) {
            if (isFile) {
                return null;
            }
            try {
                File toAdjust2 = new File(spec);
                if (toAdjust2.isDirectory()) {
                    return adjustTrailingSlash(toAdjust2.toURL(), trailingSlash);
                }
                return toAdjust2.toURL();
            } catch (MalformedURLException unused2) {
                return null;
            }
        }
    }

    private static URL adjustTrailingSlash(URL url, boolean trailingSlash) throws MalformedURLException {
        String file = url.getFile();
        if (trailingSlash == file.endsWith("/")) {
            return url;
        }
        return new URL(url.getProtocol(), url.getHost(), trailingSlash ? String.valueOf(file) + "/" : file.substring(0, file.length() - 1));
    }

    private URL buildLocation(String property, URL defaultLocation, String userDefaultAppendage) {
        URL result;
        URL result2 = null;
        String location = System.getProperty(property);
        System.getProperties().remove(property);
        if (location == null) {
            result = defaultLocation;
        } else {
            try {
                if (location.equalsIgnoreCase(NONE)) {
                }
                if (location.equalsIgnoreCase(NO_DEFAULT)) {
                    result = buildURL(location, true);
                } else {
                    if (location.startsWith(USER_HOME)) {
                        String base = substituteVar(location, USER_HOME, PROP_USER_HOME);
                        location = new File(base, userDefaultAppendage).getAbsolutePath();
                    } else if (location.startsWith(USER_DIR)) {
                        String base2 = substituteVar(location, USER_DIR, PROP_USER_DIR);
                        location = new File(base2, userDefaultAppendage).getAbsolutePath();
                    }
                    result = buildURL(location, true);
                }
            } finally {
                if (0 != 0) {
                    System.getProperties().put(property, result2.toExternalForm());
                }
            }
        }
        if (result != null) {
            System.getProperties().put(property, result.toExternalForm());
        }
        return result;
    }

    private String substituteVar(String source, String var, String prop) {
        String value = System.getProperty(prop, "");
        return String.valueOf(value) + source.substring(var.length());
    }

    private String computeDefaultConfigurationLocation() {
        URL install = getInstallLocation();
        if (install.getProtocol().equals("file")) {
            File installDir = new File(install.getFile());
            if (canWrite(installDir)) {
                return String.valueOf(installDir.getAbsolutePath()) + File.separator + CONFIG_DIR;
            }
        }
        return computeDefaultUserAreaLocation(CONFIG_DIR);
    }

    private static boolean canWrite(File installDir) {
        if (!installDir.canWrite() || !installDir.isDirectory()) {
            return false;
        }
        File fileTest = null;
        try {
            fileTest = File.createTempFile("writtableArea", ".dll", installDir);
            if (fileTest != null) {
                fileTest.delete();
                return true;
            }
            return true;
        } catch (IOException unused) {
            if (fileTest == null) {
                return false;
            }
            fileTest.delete();
            return false;
        } catch (Throwable th) {
            if (fileTest != null) {
                fileTest.delete();
            }
            throw th;
        }
    }

    private String computeDefaultUserAreaLocation(String pathAppendage) {
        int hashCode;
        URL installURL = getInstallLocation();
        if (installURL == null) {
            return null;
        }
        File installDir = new File(installURL.getFile());
        try {
            hashCode = installDir.getCanonicalPath().hashCode();
        } catch (IOException unused) {
            hashCode = installDir.getAbsolutePath().hashCode();
        }
        if (hashCode < 0) {
            hashCode = -hashCode;
        }
        String installDirHash = String.valueOf(hashCode);
        String appName = ".eclipse";
        File eclipseProduct = new File(installDir, PRODUCT_SITE_MARKER);
        if (eclipseProduct.exists()) {
            Properties props = new Properties();
            try {
                props.load(new FileInputStream(eclipseProduct));
                String appId = props.getProperty(PRODUCT_SITE_ID);
                if (appId == null || appId.trim().length() == 0) {
                    appId = ECLIPSE;
                }
                String appVersion = props.getProperty(PRODUCT_SITE_VERSION);
                if (appVersion == null || appVersion.trim().length() == 0) {
                    appVersion = "";
                }
                appName = String.valueOf(appName) + File.separator + appId + "_" + appVersion + "_" + installDirHash;
            } catch (IOException unused2) {
                appName = String.valueOf(appName) + File.separator + installDirHash;
            }
        } else {
            appName = String.valueOf(appName) + File.separator + installDirHash;
        }
        String userHome = System.getProperty(PROP_USER_HOME);
        return new File(userHome, String.valueOf(appName) + "/" + pathAppendage).getAbsolutePath();
    }

    public static void main(String argString) {
        Vector list = new Vector(5);
        StringTokenizer tokens = new StringTokenizer(argString, " ");
        while (tokens.hasMoreElements()) {
            list.addElement(tokens.nextElement());
        }
        main((String[]) list.toArray(new String[list.size()]));
    }

    public static void main(String[] args) {
        try {
            new Main().run(args);
        } catch (Throwable t) {
            t.printStackTrace();
        }
    }

    public int run(String[] args) {
        int result;
        String message;
        int parseInt;
        try {
            try {
                basicRun(args);
                String exitCode = System.getProperty(PROP_EXITCODE);
                if (exitCode == null) {
                    parseInt = 0;
                } else {
                    try {
                        parseInt = Integer.parseInt(exitCode);
                    } catch (NumberFormatException unused) {
                        result = 17;
                    }
                }
                result = parseInt;
                takeDownSplash();
                if (this.bridge != null) {
                    this.bridge.uninitialize();
                }
            } catch (Throwable th) {
                takeDownSplash();
                if (this.bridge != null) {
                    this.bridge.uninitialize();
                }
                throw th;
            }
        } catch (Throwable e) {
            if (!"13".equals(System.getProperty(PROP_EXITCODE))) {
                log("Exception launching the Eclipse Platform:");
                log(e);
                if (this.logFile == null) {
                    message = String.valueOf("An error has occurred") + " and could not be logged: \n" + e.getMessage();
                } else {
                    message = String.valueOf("An error has occurred") + ".  See the log file\n" + this.logFile.getAbsolutePath();
                }
                System.getProperties().put(PROP_EXITDATA, message);
            }
            result = 13;
            takeDownSplash();
            if (this.bridge != null) {
                this.bridge.uninitialize();
            }
        }
        System.getProperties().put(PROP_EXITCODE, Integer.toString(result));
        setExitData();
        return result;
    }

    private void setExitData() {
        String data = System.getProperty(PROP_EXITDATA);
        if (data == null || this.bridge == null) {
            return;
        }
        this.bridge.setExitData(this.exitData, data);
    }

    protected String[] processCommandLine(String[] args) {
        if (args.length == 0) {
            return args;
        }
        int[] configArgs = new int[args.length];
        configArgs[0] = -1;
        int configArgIndex = 0;
        int i = 0;
        while (i < args.length) {
            boolean found = false;
            if (args[i].equalsIgnoreCase(DEBUG)) {
                this.debug = true;
            } else {
                if (args[i].equalsIgnoreCase(NOSPLASH)) {
                    this.splashDown = true;
                    found = true;
                }
                if (args[i].equalsIgnoreCase(NOEXIT)) {
                    System.getProperties().put(PROP_NOSHUTDOWN, "true");
                    found = true;
                }
                if (args[i].equalsIgnoreCase(INITIALIZE)) {
                    this.initialize = true;
                } else if (args[i].equalsIgnoreCase(DEV) && (i + 1 == args.length || (i + 1 < args.length && args[i + 1].startsWith("-")))) {
                    this.inDevelopmentMode = true;
                } else {
                    if (args[i].equalsIgnoreCase(SHOWSPLASH)) {
                        this.showSplash = true;
                        found = true;
                        if (i + 1 < args.length && !args[i + 1].startsWith("-")) {
                            int i2 = configArgIndex;
                            configArgIndex++;
                            int i3 = i;
                            i++;
                            configArgs[i2] = i3;
                            this.splashLocation = args[i];
                        }
                    }
                    if (found) {
                        int i4 = configArgIndex;
                        configArgIndex++;
                        configArgs[i4] = i;
                    } else if (args[i].equalsIgnoreCase(VMARGS)) {
                        args[i] = null;
                        i++;
                        this.vmargs = new String[args.length - i];
                        int j = 0;
                        while (i < args.length) {
                            int i5 = j;
                            j++;
                            this.vmargs[i5] = args[i];
                            args[i] = null;
                            i++;
                        }
                    } else if (i != args.length - 1 && !args[i + 1].startsWith("-")) {
                        i++;
                        String arg = args[i];
                        if (args[i - 1].equalsIgnoreCase(DEV)) {
                            this.inDevelopmentMode = true;
                            this.devClassPathProps = processDevArg(arg);
                            if (this.devClassPathProps != null) {
                                this.devClassPath = this.devClassPathProps.getProperty(OSGI);
                                if (this.devClassPath == null) {
                                    this.devClassPath = this.devClassPathProps.getProperty("*");
                                }
                            }
                        } else {
                            if (args[i - 1].equalsIgnoreCase(FRAMEWORK)) {
                                this.framework = arg;
                                found = true;
                            }
                            if (args[i - 1].equalsIgnoreCase(OS)) {
                                this.os = arg;
                                System.getProperties().put(PROP_OS_TYPE, this.os);
                            } else if (args[i - 1].equalsIgnoreCase(WS)) {
                                this.ws = arg;
                            } else if (args[i - 1].equalsIgnoreCase(ARCH)) {
                                this.arch = arg;
                            } else {
                                if (args[i - 1].equalsIgnoreCase(INSTALL)) {
                                    System.getProperties().put(PROP_INSTALL_AREA, arg);
                                    found = true;
                                }
                                if (args[i - 1].equalsIgnoreCase(CONFIGURATION)) {
                                    System.getProperties().put(PROP_CONFIG_AREA, arg);
                                    found = true;
                                }
                                if (args[i - 1].equalsIgnoreCase(EXITDATA)) {
                                    this.exitData = arg;
                                    found = true;
                                }
                                if (args[i - 1].equalsIgnoreCase(NAME)) {
                                    found = true;
                                }
                                if (args[i - 1].equalsIgnoreCase(STARTUP)) {
                                    found = true;
                                }
                                if (args[i - 1].equalsIgnoreCase(LAUNCHER)) {
                                    found = true;
                                }
                                if (args[i - 1].equalsIgnoreCase(LIBRARY)) {
                                    this.library = arg;
                                    found = true;
                                }
                                if (args[i - 1].equalsIgnoreCase(ENDSPLASH)) {
                                    this.endSplash = arg;
                                    found = true;
                                }
                                if (args[i - 1].equalsIgnoreCase(VM)) {
                                    this.vm = arg;
                                    found = true;
                                }
                                if (args[i - 1].equalsIgnoreCase(NL)) {
                                    System.getProperties().put(PROP_NL, arg);
                                    found = true;
                                }
                                if (found) {
                                    int i6 = configArgIndex;
                                    int configArgIndex2 = configArgIndex + 1;
                                    configArgs[i6] = i - 1;
                                    configArgIndex = configArgIndex2 + 1;
                                    configArgs[configArgIndex2] = i;
                                }
                            }
                        }
                    }
                }
            }
            i++;
        }
        String[] passThruArgs = new String[(args.length - configArgIndex) - (this.vmargs == null ? 0 : this.vmargs.length + 1)];
        int configArgIndex3 = 0;
        int j2 = 0;
        for (int i7 = 0; i7 < args.length; i7++) {
            if (i7 == configArgs[configArgIndex3]) {
                configArgIndex3++;
            } else if (args[i7] != null) {
                int i8 = j2;
                j2++;
                passThruArgs[i8] = args[i7];
            }
        }
        return passThruArgs;
    }

    private Properties processDevArg(String arg) {
        if (arg == null) {
            return null;
        }
        try {
            URL location = new URL(arg);
            return load(location, null);
        } catch (MalformedURLException unused) {
            Properties result = new Properties();
            result.put("*", arg);
            return result;
        } catch (IOException unused2) {
            return null;
        }
    }

    private URL getConfigurationLocation() {
        if (this.configurationLocation != null) {
            return this.configurationLocation;
        }
        this.configurationLocation = buildLocation(PROP_CONFIG_AREA, null, "");
        if (this.configurationLocation == null) {
            this.configurationLocation = buildLocation(PROP_CONFIG_AREA_DEFAULT, null, "");
            if (this.configurationLocation == null) {
                this.configurationLocation = buildURL(computeDefaultConfigurationLocation(), true);
            }
        }
        if (this.configurationLocation != null) {
            System.getProperties().put(PROP_CONFIG_AREA, this.configurationLocation.toExternalForm());
        }
        if (this.debug) {
            System.out.println("Configuration location:\n    " + this.configurationLocation);
        }
        return this.configurationLocation;
    }

    private void processConfiguration() {
        URL baseConfigurationLocation = null;
        Properties baseConfiguration = null;
        if (System.getProperty(PROP_CONFIG_AREA) == null) {
            String baseLocation = System.getProperty(PROP_BASE_CONFIG_AREA);
            if (baseLocation != null) {
                baseConfigurationLocation = buildURL(baseLocation, true);
            }
            if (baseConfigurationLocation == null) {
                try {
                    baseConfigurationLocation = new URL(getInstallLocation(), CONFIG_DIR);
                } catch (MalformedURLException unused) {
                }
            }
            baseConfiguration = loadConfiguration(baseConfigurationLocation);
            if (baseConfiguration != null) {
                String location = baseConfiguration.getProperty(PROP_CONFIG_AREA);
                if (location != null) {
                    System.getProperties().put(PROP_CONFIG_AREA, location);
                }
                String location2 = baseConfiguration.getProperty(PROP_INSTALL_AREA);
                if (location2 != null && System.getProperty(PROP_INSTALL_AREA) == null) {
                    System.getProperties().put(PROP_INSTALL_AREA, location2);
                }
            }
        }
        Properties configuration = baseConfiguration;
        if (configuration == null || !getConfigurationLocation().equals(baseConfigurationLocation)) {
            configuration = loadConfiguration(getConfigurationLocation());
        }
        mergeProperties(System.getProperties(), configuration);
        if ("false".equalsIgnoreCase(System.getProperty(PROP_CONFIG_CASCADED))) {
            System.getProperties().remove(PROP_SHARED_CONFIG_AREA);
        } else {
            ensureAbsolute(PROP_SHARED_CONFIG_AREA);
            URL sharedConfigURL = buildLocation(PROP_SHARED_CONFIG_AREA, null, "");
            if (sharedConfigURL == null) {
                try {
                    sharedConfigURL = new URL(getInstallLocation(), CONFIG_DIR);
                } catch (MalformedURLException unused2) {
                }
            }
            if (sharedConfigURL != null) {
                if (sharedConfigURL.equals(getConfigurationLocation())) {
                    System.getProperties().remove(PROP_SHARED_CONFIG_AREA);
                } else {
                    Properties configuration2 = baseConfiguration;
                    if (!sharedConfigURL.equals(baseConfigurationLocation)) {
                        configuration2 = loadConfiguration(sharedConfigURL);
                    }
                    mergeProperties(System.getProperties(), configuration2);
                    System.getProperties().put(PROP_SHARED_CONFIG_AREA, sharedConfigURL.toExternalForm());
                    if (this.debug) {
                        System.out.println("Shared configuration location:\n    " + sharedConfigURL.toExternalForm());
                    }
                }
            }
        }
        String urlString = System.getProperty(PROP_FRAMEWORK, null);
        if (urlString != null) {
            URL url = buildURL(urlString, true);
            System.getProperties().put(PROP_FRAMEWORK, url.toExternalForm());
            this.bootLocation = resolve(urlString);
        }
    }

    private void ensureAbsolute(String locationProperty) {
        String propertyValue = System.getProperty(locationProperty);
        if (propertyValue == null) {
            return;
        }
        try {
            URL locationURL = new URL(propertyValue);
            String locationPath = locationURL.getPath();
            if (locationPath.startsWith("/")) {
                return;
            }
            URL installURL = getInstallLocation();
            if (!locationURL.getProtocol().equals(installURL.getProtocol())) {
                return;
            }
            try {
                URL absoluteURL = new URL(installURL, locationPath);
                System.getProperties().put(locationProperty, absoluteURL.toExternalForm());
            } catch (MalformedURLException unused) {
            }
        } catch (MalformedURLException unused2) {
        }
    }

    private URL getInstallLocation() {
        if (this.installLocation != null) {
            return this.installLocation;
        }
        String installArea = System.getProperty(PROP_INSTALL_AREA);
        if (installArea != null) {
            this.installLocation = buildURL(installArea, true);
            if (this.installLocation == null) {
                throw new IllegalStateException("Install location is invalid: " + installArea);
            }
            System.getProperties().put(PROP_INSTALL_AREA, this.installLocation.toExternalForm());
            if (this.debug) {
                System.out.println("Install location:\n    " + this.installLocation);
            }
            return this.installLocation;
        }
        ProtectionDomain domain = Main.class.getProtectionDomain();
        CodeSource source = null;
        URL result = null;
        if (domain != null) {
            source = domain.getCodeSource();
        }
        if (source == null || domain == null) {
            if (this.debug) {
                System.out.println("CodeSource location is null. Defaulting the install location to file:startup.jar");
            }
            try {
                result = new URL("file:startup.jar");
            } catch (MalformedURLException unused) {
            }
        }
        if (source != null) {
            result = source.getLocation();
        }
        File file = new File(decode(result.getFile()));
        String path = file.toString().replace('\\', '/');
        if (File.separatorChar == '\\' && Character.isUpperCase(path.charAt(0))) {
            char[] chars = path.toCharArray();
            chars[0] = Character.toLowerCase(chars[0]);
            path = new String(chars);
        }
        System.setProperty("com.jiuqi.dna.launcher.path", file.getAbsolutePath());
        if (path.toLowerCase().endsWith(".jar")) {
            path = path.substring(0, path.lastIndexOf("/") + 1);
        }
        if (path.toLowerCase().endsWith("/bundles/")) {
            path = path.substring(0, path.length() - "/bundles/".length());
        } else if (path.toLowerCase().endsWith("/thr/")) {
            path = path.substring(0, path.length() - "/thr/".length());
        }
        if (path.contains("#") || path.contains("%") || path.contains("!")) {
            System.err.println("Install path: " + path);
            System.err.println("ERROR: Install path contains illegal characters like \"#%!\".");
            System.exit(1);
        }
        try {
            try {
                path = new File(path).toURL().getFile();
            } catch (MalformedURLException unused2) {
            }
        } catch (MalformedURLException unused3) {
        }
        this.installLocation = new URL(result.getProtocol(), result.getHost(), result.getPort(), path);
        System.getProperties().put(PROP_INSTALL_AREA, this.installLocation.toExternalForm());
        if (this.debug) {
            System.out.println("Install location:\n    " + this.installLocation);
        }
        return this.installLocation;
    }

    private Properties loadConfiguration(URL url) {
        Properties result = null;
        try {
            URL url2 = new URL(url, CONFIG_FILE);
            try {
                if (this.debug) {
                    System.out.print("Configuration file:\n    " + url2.toString());
                }
                result = loadProperties(url2);
                if (this.debug) {
                    System.out.println(" loaded");
                }
            } catch (IOException unused) {
                if (this.debug) {
                    System.out.println(" not found or not read");
                }
            }
            return result;
        } catch (MalformedURLException unused2) {
            return null;
        }
    }

    private Properties loadProperties(URL url) throws IOException {
        Properties result;
        if (url == null) {
            return null;
        }
        try {
            result = load(url, null);
        } catch (IOException e1) {
            try {
                result = load(url, CONFIG_FILE_TEMP_SUFFIX);
            } catch (IOException unused) {
                try {
                    result = load(url, CONFIG_FILE_BAK_SUFFIX);
                } catch (IOException unused2) {
                    throw e1;
                }
            }
        }
        return result;
    }

    private Properties load(URL url, String suffix) throws IOException {
        if (suffix != null && !suffix.equals("")) {
            url = new URL(url.getProtocol(), url.getHost(), url.getPort(), String.valueOf(url.getFile()) + suffix);
        }
        Properties props = new Properties();
        InputStream is = null;
        try {
            is = url.openStream();
            props.load(is);
            if (is != null) {
                try {
                    is.close();
                } catch (IOException unused) {
                }
            }
            return props;
        } catch (Throwable th) {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException unused2) {
                }
            }
            throw th;
        }
    }

    private void handleSplash(URL[] defaultPath) {
        if (this.initialize || this.splashDown || this.bridge == null) {
            this.showSplash = false;
            this.endSplash = null;
            return;
        }
        if (this.showSplash || this.endSplash != null) {
            try {
                Runtime.getRuntime().addShutdownHook(this.splashHandler);
            } catch (Throwable unused) {
            }
        }
        if (this.endSplash != null) {
            this.showSplash = false;
            return;
        }
        if (!this.showSplash) {
            return;
        }
        this.splashLocation = getSplashLocation(defaultPath);
        if (this.debug) {
            System.out.println("Splash location:\n    " + this.splashLocation);
        }
        if (this.splashLocation == null) {
            return;
        }
        this.bridge.showSplash(this.splashLocation);
        long handle = this.bridge.getSplashHandle();
        if (handle != 0 && handle != -1) {
            System.getProperties().put(SPLASH_HANDLE, String.valueOf(handle));
            System.getProperties().put(SPLASH_LOCATION, this.splashLocation);
            this.bridge.updateSplash();
        }
    }

    protected void takeDownSplash() {
        if (this.splashDown || this.bridge == null) {
            return;
        }
        this.splashDown = this.bridge.takeDownSplash();
        System.getProperties().remove(SPLASH_HANDLE);
        try {
            Runtime.getRuntime().removeShutdownHook(this.splashHandler);
        } catch (Throwable unused) {
        }
    }

    private String getSplashLocation(URL[] bootPath) {
        if (this.splashLocation != null && !Character.isDigit(this.splashLocation.charAt(0)) && new File(this.splashLocation).exists()) {
            System.getProperties().put(PROP_SPLASHLOCATION, this.splashLocation);
            return this.splashLocation;
        }
        String result = System.getProperty(PROP_SPLASHLOCATION);
        if (result != null) {
            return result;
        }
        String splashPath = System.getProperty(PROP_SPLASHPATH);
        if (splashPath != null) {
            String[] entries = getArrayFromList(splashPath);
            ArrayList path = new ArrayList(entries.length);
            for (int i = 0; i < entries.length; i++) {
                String entry = resolve(entries[i]);
                if (entry == null || entry.startsWith(FILE_SCHEME)) {
                    File entryFile = new File(entry.substring(5).replace('/', File.separatorChar));
                    String entry2 = searchFor(entryFile.getName(), entryFile.getParent());
                    if (entry2 != null) {
                        path.add(entry2);
                    }
                } else {
                    log("Invalid splash path entry: " + entries[i]);
                }
            }
            String result2 = searchForSplash((String[]) path.toArray(new String[path.size()]));
            if (result2 != null) {
                System.getProperties().put(PROP_SPLASHLOCATION, result2);
                return result2;
            }
        }
        String result3 = searchSplash(bootPath, "bundles");
        if (result3 == null) {
            result3 = searchSplash(bootPath, "thr");
        }
        return result3;
    }

    private String searchSplash(URL[] bootPath, String dir) {
        int pix;
        String result = null;
        String temp = bootPath[0].getFile().replace('/', File.separatorChar);
        int ix = temp.lastIndexOf(String.valueOf(dir) + File.separator);
        if (ix != -1 && (pix = temp.indexOf(File.separator, ix + 8)) != -1) {
            result = searchForSplash(new String[]{temp.substring(0, pix)});
            if (result != null) {
                System.getProperties().put(PROP_SPLASHLOCATION, result);
            }
        }
        return result;
    }

    private String searchForSplash(String[] searchPath) {
        if (searchPath == null) {
            return null;
        }
        String locale = (String) System.getProperties().get(PROP_NL);
        if (locale == null) {
            locale = Locale.getDefault().toString();
        }
        String[] nlVariants = buildNLVariants(locale);
        for (int i = 0; i < nlVariants.length; i++) {
            for (int j = 0; j < searchPath.length; j++) {
                if (isJAR(searchPath[j])) {
                    String result = extractSplashFromJAR(searchPath[j], nlVariants[i]);
                    if (result != null) {
                        return result;
                    }
                } else {
                    String path = searchPath[j];
                    if (!path.endsWith(File.separator)) {
                        path = String.valueOf(path) + File.separator;
                    }
                    File result2 = new File(String.valueOf(path) + nlVariants[i]);
                    if (result2.exists()) {
                        return result2.getAbsolutePath();
                    }
                }
            }
        }
        return null;
    }

    private static void transferStreams(InputStream source, OutputStream destination) {
        byte[] buffer = new byte[8096];
        while (true) {
            try {
                try {
                    int bytesRead = source.read(buffer);
                    if (bytesRead == -1) {
                        try {
                            source.close();
                            try {
                                destination.close();
                                return;
                            } catch (IOException unused) {
                                return;
                            }
                        } catch (IOException unused2) {
                            try {
                                destination.close();
                                return;
                            } catch (IOException unused3) {
                                return;
                            }
                        } catch (Throwable th) {
                            try {
                                destination.close();
                            } catch (IOException unused4) {
                            }
                            throw th;
                        }
                    }
                    try {
                        destination.write(buffer, 0, bytesRead);
                    } catch (IOException unused5) {
                        try {
                            source.close();
                            try {
                                destination.close();
                                return;
                            } catch (IOException unused6) {
                                return;
                            }
                        } catch (IOException unused7) {
                            try {
                                destination.close();
                                return;
                            } catch (IOException unused8) {
                                return;
                            }
                        } catch (Throwable th2) {
                            try {
                                destination.close();
                            } catch (IOException unused9) {
                            }
                            throw th2;
                        }
                    }
                } catch (IOException unused10) {
                    try {
                        source.close();
                        try {
                            destination.close();
                            return;
                        } catch (IOException unused11) {
                            return;
                        }
                    } catch (IOException unused12) {
                        try {
                            destination.close();
                            return;
                        } catch (IOException unused13) {
                            return;
                        }
                    } catch (Throwable th3) {
                        try {
                            destination.close();
                        } catch (IOException unused14) {
                        }
                        throw th3;
                    }
                }
            } catch (Throwable th4) {
                try {
                    source.close();
                    try {
                        destination.close();
                    } catch (IOException unused15) {
                    }
                } catch (IOException unused16) {
                    try {
                        destination.close();
                    } catch (IOException unused17) {
                    }
                } catch (Throwable th5) {
                    try {
                        destination.close();
                    } catch (IOException unused18) {
                    }
                    throw th5;
                }
                throw th4;
            }
        }
    }

    private String extractSplashFromJAR(String jarPath, String splashPath) {
        String configLocation = System.getProperty(PROP_CONFIG_AREA);
        if (configLocation == null) {
            log("Configuration area not set yet. Unable to extract splash from JAR'd plug-in: " + jarPath);
            return null;
        }
        URL configURL = buildURL(configLocation, false);
        if (configURL == null) {
            return null;
        }
        File splash = new File(configURL.getPath(), PLUGIN_ID);
        File jarFile = new File(jarPath);
        String cache = jarFile.getName();
        if (cache.endsWith(".jar")) {
            cache = cache.substring(0, cache.length() - ERROR);
        }
        File splash2 = new File(new File(splash, cache), splashPath);
        if (splash2.exists()) {
            boolean clean = false;
            int i = 0;
            while (true) {
                if (i >= this.commands.length) {
                    break;
                }
                if (!CLEAN.equalsIgnoreCase(this.commands[i])) {
                    i++;
                } else {
                    clean = true;
                    splash2.delete();
                    break;
                }
            }
            if (!clean) {
                return splash2.getAbsolutePath();
            }
        }
        try {
            ZipFile file = new ZipFile(jarPath);
            ZipEntry entry = file.getEntry(splashPath.replace(File.separatorChar, '/'));
            if (entry == null) {
                return null;
            }
            try {
                InputStream input = file.getInputStream(entry);
                new File(splash2.getParent()).mkdirs();
                try {
                    OutputStream output = new BufferedOutputStream(new FileOutputStream(splash2));
                    transferStreams(input, output);
                    if (splash2.exists()) {
                        return splash2.getAbsolutePath();
                    }
                    return null;
                } catch (FileNotFoundException unused) {
                    try {
                        input.close();
                        return null;
                    } catch (IOException unused2) {
                        return null;
                    }
                }
            } catch (IOException e) {
                log("Exception opening splash: " + entry.getName() + " in JAR file: " + jarPath);
                log(e);
                return null;
            }
        } catch (IOException e2) {
            log("Exception looking for splash  in JAR file: " + jarPath);
            log(e2);
            return null;
        }
    }

    private boolean isJAR(String path) {
        int index;
        if (path.endsWith(File.separator)) {
            return false;
        }
        int index2 = path.lastIndexOf(46);
        if (index2 == -1 || (index = index2 + 1) >= path.length()) {
            return false;
        }
        return "JAR".equalsIgnoreCase(path.substring(index));
    }

    private static String[] buildNLVariants(String locale) {
        String nl = locale;
        ArrayList result = new ArrayList(ERROR);
        while (true) {
            result.add("nl" + File.separatorChar + nl.replace('_', File.separatorChar) + File.separatorChar + SPLASH_IMAGE);
            int lastSeparator = nl.lastIndexOf(95);
            if (lastSeparator != -1) {
                nl = nl.substring(0, lastSeparator);
            } else {
                result.add(SPLASH_IMAGE);
                return (String[]) result.toArray(new String[result.size()]);
            }
        }
    }

    private String resolve(String urlString) {
        if (urlString.startsWith(REFERENCE_SCHEME)) {
            urlString = urlString.substring(10);
            System.getProperties().put(PROP_FRAMEWORK, urlString);
        }
        if (urlString.startsWith(PLATFORM_URL)) {
            String path = urlString.substring(PLATFORM_URL.length());
            return getInstallLocation() + path;
        }
        return urlString;
    }

    /* JADX WARN: Finally extract failed */
    protected synchronized void log(Object obj) {
        try {
            if (obj == null) {
                return;
            }
            try {
                openLogFile();
                try {
                    if (this.newSession) {
                        this.log.write(SESSION);
                        this.log.write(32);
                        String timestamp = new Date().toString();
                        this.log.write(timestamp);
                        this.log.write(32);
                        for (int i = SESSION.length() + timestamp.length(); i < 78; i++) {
                            this.log.write(45);
                        }
                        this.log.newLine();
                        this.newSession = false;
                    }
                    write(obj);
                    if (this.logFile == null) {
                        if (this.log != null) {
                            this.log.flush();
                        }
                    } else {
                        closeLogFile();
                    }
                    this.log = null;
                } catch (Throwable th) {
                    if (this.logFile == null) {
                        if (this.log != null) {
                            this.log.flush();
                        }
                    } else {
                        closeLogFile();
                    }
                    throw th;
                }
            } catch (Exception e) {
                System.err.println("An exception occurred while writing to the platform log:");
                e.printStackTrace(System.err);
                System.err.println("Logging to the console instead.");
                try {
                    this.log = logForStream(System.err);
                    write(obj);
                    this.log.flush();
                } catch (Exception e2) {
                    System.err.println("An exception occurred while logging to the console:");
                    e2.printStackTrace(System.err);
                }
                this.log = null;
            }
        } catch (Throwable th2) {
            this.log = null;
            throw th2;
        }
    }

    private void write(Object obj) throws IOException {
        if (obj == null) {
            return;
        }
        if (obj instanceof Throwable) {
            this.log.write(STACK);
            this.log.newLine();
            ((Throwable) obj).printStackTrace(new PrintWriter(this.log));
        } else {
            this.log.write(ENTRY);
            this.log.write(32);
            this.log.write(PLUGIN_ID);
            this.log.write(32);
            this.log.write(String.valueOf(ERROR));
            this.log.write(32);
            this.log.write(String.valueOf(0));
            this.log.write(32);
            this.log.write(getDate(new Date()));
            this.log.newLine();
            this.log.write(MESSAGE);
            this.log.write(32);
            this.log.write(String.valueOf(obj));
        }
        this.log.newLine();
    }

    protected String getDate(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        StringBuffer sb = new StringBuffer();
        appendPaddedInt(c.get(1), ERROR, sb).append('-');
        appendPaddedInt(c.get(2) + 1, 2, sb).append('-');
        appendPaddedInt(c.get(5), 2, sb).append(' ');
        appendPaddedInt(c.get(11), 2, sb).append(':');
        appendPaddedInt(c.get(12), 2, sb).append(':');
        appendPaddedInt(c.get(13), 2, sb).append('.');
        appendPaddedInt(c.get(14), 3, sb);
        return sb.toString();
    }

    private StringBuffer appendPaddedInt(int value, int pad, StringBuffer buffer) {
        int pad2 = pad - 1;
        if (pad2 == 0) {
            return buffer.append(Integer.toString(value));
        }
        int padding = (int) Math.pow(10.0d, pad2);
        if (value >= padding) {
            return buffer.append(Integer.toString(value));
        }
        while (padding > value && padding > 1) {
            buffer.append('0');
            padding /= 10;
        }
        buffer.append(value);
        return buffer;
    }

    private void computeLogFileLocation() {
        String logFileProp = System.getProperty(PROP_LOGFILE);
        if (logFileProp != null) {
            if (this.logFile == null || !logFileProp.equals(this.logFile.getAbsolutePath())) {
                this.logFile = new File(logFileProp);
                new File(this.logFile.getParent()).mkdirs();
                return;
            }
            return;
        }
        URL base = buildURL(System.getProperty(PROP_CONFIG_AREA), false);
        if (base == null) {
            return;
        }
        this.logFile = new File(base.getPath(), String.valueOf(Long.toString(System.currentTimeMillis())) + ".log");
        new File(this.logFile.getParent()).mkdirs();
        System.getProperties().put(PROP_LOGFILE, this.logFile.getAbsolutePath());
    }

    private int hexToByte(byte b) {
        switch (b) {
            case 48:
                return 0;
            case 49:
                return 1;
            case 50:
                return 2;
            case 51:
                return 3;
            case 52:
                return ERROR;
            case 53:
                return 5;
            case 54:
                return 6;
            case 55:
                return 7;
            case 56:
                return 8;
            case 57:
                return 9;
            case 58:
            case 59:
            case 60:
            case 61:
            case 62:
            case 63:
            case 64:
            case 71:
            case 72:
            case 73:
            case 74:
            case 75:
            case 76:
            case 77:
            case 78:
            case 79:
            case 80:
            case 81:
            case 82:
            case 83:
            case 84:
            case 85:
            case 86:
            case 87:
            case 88:
            case 89:
            case 90:
            case 91:
            case 92:
            case 93:
            case 94:
            case 95:
            case 96:
            default:
                throw new IllegalArgumentException("Switch error decoding URL");
            case 65:
            case 97:
                return 10;
            case 66:
            case 98:
                return 11;
            case 67:
            case 99:
                return 12;
            case 68:
            case 100:
                return 13;
            case 69:
            case 101:
                return 14;
            case 70:
            case 102:
                return 15;
        }
    }

    private void openLogFile() throws IOException {
        computeLogFileLocation();
        try {
            this.log = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(this.logFile.getAbsolutePath(), true), "UTF-8"));
        } catch (IOException e) {
            this.logFile = null;
            throw e;
        }
    }

    private BufferedWriter logForStream(OutputStream output) {
        try {
            return new BufferedWriter(new OutputStreamWriter(output, "UTF-8"));
        } catch (UnsupportedEncodingException unused) {
            return new BufferedWriter(new OutputStreamWriter(output));
        }
    }

    private void closeLogFile() throws IOException {
        try {
            if (this.log != null) {
                this.log.flush();
                this.log.close();
            }
        } finally {
            this.log = null;
        }
    }

    private void mergeProperties(Properties destination, Properties source) {
        String destinationClasspath;
        if (destination == null || source == null) {
            return;
        }
        Enumeration e = source.keys();
        while (e.hasMoreElements()) {
            String key = (String) e.nextElement();
            if (key.equals(PROP_CLASSPATH)) {
                String destinationClasspath2 = destination.getProperty(PROP_CLASSPATH);
                String sourceClasspath = source.getProperty(PROP_CLASSPATH);
                if (destinationClasspath2 == null) {
                    destinationClasspath = sourceClasspath;
                } else {
                    destinationClasspath = String.valueOf(destinationClasspath2) + sourceClasspath;
                }
                destination.put(PROP_CLASSPATH, destinationClasspath);
            } else {
                String value = source.getProperty(key);
                if (destination.getProperty(key) == null) {
                    destination.put(key, value);
                }
            }
        }
    }

    private void setupVMProperties() {
        if (this.vm != null) {
            System.getProperties().put(PROP_VM, this.vm);
        }
        setMultiValueProperty(PROP_VMARGS, this.vmargs);
        setMultiValueProperty(PROP_COMMANDS, this.commands);
    }

    private void setMultiValueProperty(String property, String[] value) {
        if (value != null) {
            StringBuffer result = new StringBuffer(300);
            for (int i = 0; i < value.length; i++) {
                if (value[i] != null) {
                    result.append(value[i]);
                    result.append('\n');
                }
            }
            System.getProperties().put(property, result.toString());
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: org_eclipse_equinox_launcher.jar:org/eclipse/equinox/launcher/Main$EclipsePolicy.class */
    public class EclipsePolicy extends Policy {
        private final Policy policy;
        private final URL[] urls;
        Permission allPermission = new AllPermission();
        private final PermissionCollection allPermissions = new PermissionCollection() { // from class: org.eclipse.equinox.launcher.Main.EclipsePolicy.1
            private static final long serialVersionUID = 3258131349494708277L;

            @Override // java.security.PermissionCollection
            public void add(Permission permission) {
            }

            @Override // java.security.PermissionCollection
            public boolean implies(Permission permission) {
                return true;
            }

            @Override // java.security.PermissionCollection
            public Enumeration elements() {
                return new Enumeration() { // from class: org.eclipse.equinox.launcher.Main.EclipsePolicy.1.1
                    int cur = 0;

                    @Override // java.util.Enumeration
                    public boolean hasMoreElements() {
                        return this.cur < 1;
                    }

                    @Override // java.util.Enumeration
                    public Object nextElement() {
                        if (this.cur == 0) {
                            this.cur = 1;
                            return EclipsePolicy.this.allPermission;
                        }
                        throw new NoSuchElementException();
                    }
                };
            }
        };

        EclipsePolicy(Policy policy, URL[] urls) {
            this.policy = policy;
            this.urls = urls;
        }

        @Override // java.security.Policy
        public PermissionCollection getPermissions(CodeSource codesource) {
            if (contains(codesource.getLocation())) {
                return this.allPermissions;
            }
            return this.policy == null ? this.allPermissions : this.policy.getPermissions(codesource);
        }

        @Override // java.security.Policy
        public PermissionCollection getPermissions(ProtectionDomain domain) {
            if (contains(domain.getCodeSource().getLocation())) {
                return this.allPermissions;
            }
            return this.policy == null ? this.allPermissions : this.policy.getPermissions(domain);
        }

        @Override // java.security.Policy
        public boolean implies(ProtectionDomain domain, Permission permission) {
            if (contains(domain.getCodeSource().getLocation()) || this.policy == null) {
                return true;
            }
            return this.policy.implies(domain, permission);
        }

        @Override // java.security.Policy
        public void refresh() {
            if (this.policy != null) {
                this.policy.refresh();
            }
        }

        private boolean contains(URL url) {
            for (int i = 0; i < this.urls.length; i++) {
                if (this.urls[i] == url) {
                    return true;
                }
            }
            return false;
        }
    }

    /* JADX INFO: Access modifiers changed from: private */
    /* loaded from: org_eclipse_equinox_launcher.jar:org/eclipse/equinox/launcher/Main$StartupClassLoader.class */
    public class StartupClassLoader extends URLClassLoader {
        public StartupClassLoader(URL[] urls) {
            super(urls);
        }

        public StartupClassLoader(URL[] urls, ClassLoader parent) {
            super(urls, parent);
        }

        public StartupClassLoader(URL[] urls, ClassLoader parent, URLStreamHandlerFactory factory) {
            super(urls, parent, factory);
        }

        @Override // java.lang.ClassLoader
        protected String findLibrary(String name) {
            if (Main.this.extensionPaths == null) {
                return super.findLibrary(name);
            }
            String libName = System.mapLibraryName(name);
            for (int i = 0; i < Main.this.extensionPaths.length; i++) {
                File libFile = new File(Main.this.extensionPaths[i], libName);
                if (libFile.isFile()) {
                    return libFile.getAbsolutePath();
                }
            }
            return super.findLibrary(name);
        }
    }
}
